package importer

import (
	"crypto/rand"
	"crypto/sha256"
	"encoding/hex"
	"fmt"
	"regexp"
)

var uuid_re = regexp.MustCompile(`\A^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}\z`)

func (app *App) Run(listing Listing) error {

	// Check listing
	for _, user := range listing {
		if len(user.Email) == 0 || len(user.Fullname) == 0 || !uuid_re.MatchString(user.Uuid) {
			return fmt.Errorf("Invalid user uuid:%q email:%q fullname:%q", user.Uuid, user.Email, user.Fullname)
		}
	}

	// Send emails
	for _, user := range listing {

		userCount := 0
		err := app.Service.DB.QueryRowContext(app.Context, `SELECT COUNT(1) FROM users WHERE uuid=$1`, user.Uuid).Scan(&userCount)
		if err != nil {
			return fmt.Errorf("Error checking previous presence of user %s: %s", user.Uuid, err.Error())
		}
		if userCount == 1 {
			// User is present, and has received an email already, skip
			continue
		}
		if userCount != 0 {
			return fmt.Errorf("Unique uuid violation for %s", user.Uuid)
		}

		userPrivateTokenRaw := make([]byte, 32)
		n, err := rand.Read(userPrivateTokenRaw)
		if err != nil {
			return fmt.Errorf("Error making private token: %s", err)
		}
		if n != 32 {
			return fmt.Errorf("Error making private token: %d != 32", n)
		}
		userPrivateToken := hex.EncodeToString(userPrivateTokenRaw)
		userPublicTokenRaw := sha256.Sum256([]byte(userPrivateToken))
		userPublicToken := hex.EncodeToString(userPublicTokenRaw[:])

		tx, err := app.Service.DB.Begin()
		if err != nil {
			return fmt.Errorf("Transaction error: %s", user.Uuid, err.Error())
		}

		_, err = tx.ExecContext(app.Context, `
			INSERT INTO users(uuid, public_token, fullname, email, admin)
			VALUES($1, $2, $3, $4, $5)
		`, user.Uuid, userPublicToken, user.Fullname, user.Email, user.Admin)
		if err != nil {
			tx.Rollback()
			return fmt.Errorf("Error inserting user: %s", err)
		}

		body := fmt.Sprintf(EMAIL_TOKEN_BODY, user.Fullname, userPrivateToken)
		fmt.Println(body)
		// app.Service.Email.Send(user.Email, EMAIL_TOKEN_SUBJECT)

		err = tx.Commit()
		if err != nil {
			return fmt.Errorf("Error committing transation: %s", err)
		}

	}

	return nil
}

// TODO listing des votants pour les admins

const EMAIL_TOKEN_SUBJECT = "AG Électrolab - Votre accès au vote en ligne"

const EMAIL_TOKEN_BODY = `Bonjour %s,

En raison du confinement, l'assemblée générale de l'Électrolab a lieu sur internet. Vous pourrez suivre la présentation à cette addresse: httosL.

Vous pourrez voter sur les différents points de l'assemblée générale en vous rendant à l'adresse suivante:

https://vote.electrolab.fr/Electrolab-AG-2020/votes?private_token=%s

`
